/*
 *  Simple HTTP get webclient test
 */

#define FASTLED_ESP8266_RAW_PIN_ORDER

#include <FastLED.h>   
#include <ESP8266WiFi.h>

#define LED_PIN     13
#define NUM_LEDS    50
#define REFRESH_AFTER_MILLISEC    3600000
// once an hour

const CRGB COLOR_BLACK = CRGB(0, 0, 0);
const CRGB COLOR_RED = CRGB(10, 0, 0);
const CRGB COLOR_YELLOW = CRGB(25, 10, 0);
const CRGB COLOR_GREEN = CRGB(0, 10, 0);
const CRGB COLOR_BLUE = CRGB(0, 0, 10);
const CRGB COLOR_WHITE = CRGB(10, 10, 10);

CRGB leds[NUM_LEDS];

// Shenzen
//const char* lat     = "22.5429";
//const char* lon     = "114.0596";
// Astoria
//const char* lat     = "46.187885";
//const char* lon     = "-123.831253";
// Vancouver
const char* lat     = "49.2827";
const char* lon     = "123.1207";

const char* ssid     = "wifissid";
const char* password = "wifipw";
const char* host = "api.openweathermap.org";

void setup() {
  Serial.begin(115200);
  delay(100);

  // We start by connecting to a WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  
  WiFi.begin(ssid, password);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");  
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  Serial.print("Netmask: ");
  Serial.println(WiFi.subnetMask());
  Serial.print("Gateway: ");
  Serial.println(WiFi.gatewayIP());

  FastLED.addLeds<WS2812, LED_PIN, GRB>(leds, NUM_LEDS);
}

int value = 0;
String aqi = "1";

void loop() {
  ++value;
  
  fetchAirQualityIndex();
  setPollution(aqi.toInt());
  delay(REFRESH_AFTER_MILLISEC);
}

void fetchAirQualityIndex() {
  Serial.print("connecting to ");
  Serial.println(host);
  
  // Use WiFiClient class to create TCP connections
  WiFiClient client;
  const int httpPort = 80;
  if (!client.connect(host, httpPort)) {
    Serial.println("connection failed");
    return;
  }
  
  // We now create a URI for the request
  String url = String("/data/2.5/air_pollution?lat=") + lat + "&lon=" + lon + "&appid=yourapiid";
  Serial.print("Requesting URL: ");
  Serial.println(url);
  
  // This will send the request to the server
  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
               "Host: " + host + "\r\n" + 
               "Connection: close\r\n\r\n");
  delay(500);
  
  // Read all the lines of the reply from server and print them to Serial
  while(client.available()){
    String line = client.readStringUntil('\r');
    if (line.substring(1, 2) == "{") {
      Serial.println(String("[Response Body] ") + line);
      const int aqiIdx = line.indexOf("{\"aqi\":");
      String newAqi = line.substring(aqiIdx+7, aqiIdx+8);
      Serial.println(String("[AQI INDEX] ") + newAqi);
      aqi = newAqi;
    }
  }
  Serial.println();
  Serial.println("closing connection");
}


void setPollution(int level) {
    switch (level) {
      case 5:
        setStrip(0, COLOR_RED);
        setStrip(1, COLOR_BLACK);
        setStrip(2, COLOR_BLACK);
        setStrip(3, COLOR_BLACK);
        setStrip(4, COLOR_BLACK);
        break;
      case 4:
        setStrip(0, COLOR_YELLOW);
        setStrip(1, COLOR_YELLOW);
        setStrip(2, COLOR_BLACK);
        setStrip(3, COLOR_BLACK);
        setStrip(4, COLOR_BLACK);
        break;
      case 3:
        setStrip(0, COLOR_GREEN);
        setStrip(1, COLOR_GREEN);
        setStrip(2, COLOR_GREEN);
        setStrip(3, COLOR_BLACK);
        setStrip(4, COLOR_BLACK);
        break;
      case 2:
        setStrip(0, COLOR_BLUE);
        setStrip(1, COLOR_BLUE);
        setStrip(2, COLOR_BLUE);    
        setStrip(3, COLOR_BLUE);  
        setStrip(4, COLOR_BLACK);      
        break;
      case 1:
        setStrip(0, COLOR_WHITE);
        setStrip(1, COLOR_WHITE);
        setStrip(2, COLOR_WHITE);
        setStrip(3, COLOR_WHITE);
        setStrip(4, COLOR_WHITE);
        break;
    }
    leds[0] = COLOR_BLACK;

    FastLED.show();
}

void setStrip(int stripId, CRGB color) {
  for (int i=0;i<10;i++) {
    leds[i+stripId*10] = color;
  }
}
